mod mcdu;

use gtk::prelude::*;
use mcdu::McduModel;
use relm4::prelude::*;

struct AppModel {
    text: String,
    mcdu: Controller<McduModel>,
}

#[derive(Debug)]
enum AppInput {
    McduInput(char),
}

#[relm4::component]
impl SimpleComponent for AppModel {
    type Input = AppInput;
    type Output = ();
    type Init = ();

    view! {
        #[root]
        gtk::Window{
            set_default_width: 300,
            set_default_height: 300,
            set_title: Some("Play music!!!"),
            gtk::Box {
                set_orientation: gtk::Orientation::Horizontal,
                model.mcdu.widget(),
                #[name = "label"]
                gtk::Label {
                    #[watch]
                    set_label: &model.text,
                    set_margin_all: 5,
                }
            }
        }
    }

    fn init(
        _init: Self::Init,
        window: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> relm4::ComponentParts<Self> {
        relm4_icons::initialize_icons();

        let mcdu = McduModel::builder()
            .launch(())
            .forward(sender.input_sender(), |response| match response {
                mcdu::McduOutput::ButtonPress(c) => AppInput::McduInput(c),
            });

        let model = AppModel {
            text: String::from("Text: NONE"),
            mcdu,
        };

        let widgets = view_output!();

        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, _sender: ComponentSender<Self>) {
        match message {
            AppInput::McduInput(c) => self.text.push(c),
        }
    }
}

fn main() {
    let app = RelmApp::new("de.frajul.sheet-organizer");
    app.run::<AppModel>(());
}
